package alexsa;

import alexsa.data.*;
import alexsa.data.graphs.*;
import alexsa.data.animation.*;
import alexsa.data.tvp.*;
import alexsa.data.tvs.*;
import alexsa.gui.*;
import alexsa.gui.menus.*;

import javax.swing.*;
import javax.swing.border.EmptyBorder; 

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.FileDialog;

import java.awt.event.KeyEvent;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.FileNotFoundException;

import java.util.Vector;
import java.util.Enumeration;

public class Alexsa extends JFrame implements MenuCommander, Commands {

	class DataReader extends Thread {
		
		File
			file = null;
		JFrame
			parent = null;

		DataReader(File f, JFrame frame) {
			file = f;
			parent = frame;
		}

		public void run() {
			String fullPath = file.getPath();
			if (fullPath.indexOf('.') < 0) {
				System.err.println(fullPath + ": file not adequate");
				return;
			}
			String prefix = fullPath.substring(0, fullPath.lastIndexOf('.'));
			File
				tvp_file = new File(prefix + ".tvp"),
				trace_file = new File(prefix + ".trace"),
				graph_file = new File(prefix + ".graph"),
				tvs_file = new File(prefix + ".tvs");
			if (!tvp_file.exists()) {
				System.err.println(tvp_file.getPath() + ": file not found");
				return;
			}
			if (!trace_file.exists()) {
				System.err.println(trace_file.getPath() + ": file not found");
				return;
			}
			if (!graph_file.exists()) {
				System.err.println(graph_file.getPath() + ": file not found");
				return;
			}
			if (!tvs_file.exists()) {
				JFileChooser chooser = new JFileChooser();
				chooser.setCurrentDirectory(new File(file.getParent()));
				chooser.setFileFilter(new TvsFileFilter());
				chooser.setDialogTitle("Open TVS-file");
				if (chooser.showOpenDialog(parent) != JFileChooser.APPROVE_OPTION)
			    	return;
				tvs_file = chooser.getSelectedFile();
			}
			try {
				// read TVP-File
				messages.setText("Reading program from [" + tvp_file.getName() + "]");
	    		ProgramLoader prog = new ProgramLoader(tvp_file);
	    		Program program = prog.program;
				if (program == null) {
					messages.setText("Error while reading the file [" + tvp_file.getPath() + "]");
					return;
				}

				// read shape graphs
				messages.setText("Reading shape graphs from [" + graph_file.getName() + "]");
	    		GraphLoader graph_parser = new GraphLoader(graph_file);
				Vector graphs = graph_parser.graphs;

				// read Trace-File (contains the connections between the
				// various shape graphs)
				messages.setText("Reading trace from [" + trace_file.getName() + "]");
				Trace trace = new Trace(trace_file, graphs, program);

				// read TVS-File 
				messages.setText("Reading TVS from [" + tvs_file.getName() + "]");
				TvsLoader tvs_loader = new TvsLoader(tvs_file);
				program.tvs = tvs_loader.tvs;

				// associate the cfg with the shapes
				messages.setText("Combining shapes with program");
				program.combineNodesWithGraphs(trace);
				
				trace.createAnimations();

				// open window
				messages.setText(tvp_file.getName() + " has been read successfully");
				AlexsaWindow fw = new AlexsaWindow(program, trace);
//				fw.setBounds(10,10,700, 500);
				desktop.add(fw, JDesktopPane.DEFAULT_LAYER);
				fw.setVisible(true);
				desktop.getDesktopManager().activateFrame(fw);
				desktop.getDesktopManager().maximizeFrame(fw);
				if (program.dead_cfg_node_list != null) {
					JOptionPane.showMessageDialog(null,
						"TVLA was instructed not to print all shape graphs\n" +
						"for this file. Remove everything after the last '%%'\n"+
						"in the TVP file and rerun TVLA to get all shape graphs.\n" +
						"Until then, visual execution is not possible.",
						"Not enough shape graphs", JOptionPane.WARNING_MESSAGE, null);
				}
			} catch (FileFormatException e) {
				messages.setText("An error occured. Please see standard error for details.");
	    		System.err.println("file format exception:\n"+e.getMessage());
			} catch (Exception e2) {
				messages.setText("An error occured. Please see standard error for details.");
	    		System.err.println(e2.getMessage());
	    		e2.printStackTrace();
			}
		}
	}

	// ------------- Constants ---------------------
	private final static String
		SystemName = "Alexsa - Algorithm Explanation by Shape Analysis";
	private final static int
		NONE		= 0,
		SOME		= 1,
		NORMAL		= 2,
		HIGH		= 3,
		ALL			= 5,
		DEBUG_LEVEL = NONE;
    
    
    // ------------- Variables ---------------------
	private JDesktopPane
		desktop = null;
	private Statusbar
		messages = null;
	private FMenuBar
		menubar, invisible_menubar;

    // ------------- Constructors, Initialisation -----
	public Alexsa() {
		super(SystemName);
	
		setSize(770, 570);
		setLocation(30,30);
		
		UIManager.put("Label.foreground", java.awt.Color.black);
		UIManager.put("TitledBorder.titleColor", java.awt.Color.black);

		JPanel content = (JPanel)getContentPane();
		content.setLayout(new BorderLayout());

		desktop = new JDesktopPane();
		content.add(desktop, BorderLayout.CENTER);
		
		messages = new Statusbar();
		content.add(messages, BorderLayout.SOUTH);
		
		menubar = new StandardMenuBar(this);
		setJMenuBar(menubar);
		
		invisible_menubar = new InvisibleMenuBar(this);
		getLayeredPane().add(invisible_menubar);

		setVisible(true);
		openFile();
		menubar.updateMenus();
		invisible_menubar.updateMenus();
    }
    
    public static void main(String[] argv) {
		Preferences.readPrefs();
		Alexsa ThisAlexsa = new Alexsa();
    }
    
    
    // ---------------- Methods ----------------------
     
    public void openFile() {
		JFileChooser chooser = new JFileChooser();
		chooser.setCurrentDirectory(new File(Preferences.startup_path));
		chooser.setFileFilter(new TvpTraceGraphFileFilter());
		chooser.setDialogTitle("Open file");

		if (chooser.showOpenDialog(this) != JFileChooser.APPROVE_OPTION)
	    	return;
		
		File f = chooser.getSelectedFile();

//		File f = new File("../tvla/test/shape/SLL/DeleteAll.tvp");
		DataReader dr = new DataReader(f, this);
		dr.start();
		
    }
    
	private AlexsaWindow getFrontMostWindow() {
		JInternalFrame[] windows = desktop.getAllFrames();
		if (windows == null)
			return null;
		for (int i = 0; i < windows.length; i++)
			if (windows[i].isSelected())
				return (AlexsaWindow)windows[i];
		return null;
	}
	
    // ---------------- EventHandling ----------------
    
	public void updateMenus() {
		menubar.updateMenus();
		invisible_menubar.updateMenus();
	}

	public void doCommand(int command) {
	    switch (command) {
		    case cmd_Open:
				openFile();
				updateMenus();
				return;
	    	case cmd_Quit:
				System.exit(0);
				return;

			case cmd_PrefsAnimation:
				Preferences.changeAnimationPrefs();
				return;
			case cmd_PrefsMisc:
				Preferences.changeMiscPrefs();
				return;
			case cmd_SavePrefs:
				Preferences.savePrefs();
				return;
		}

		// the following commands only work if there is at least one
		// AlexsaWindow open
		AlexsaWindow fw = getFrontMostWindow();
		if (fw == null)
			return;
		if (fw.canHandleCommand(command))
			fw.handleCommand(command);
		else {
			switch (command) {		
		    	case cmd_Close:
					fw.dispose();
					break;
				case cmd_ViewDeclarations:
					break;
			}
		}
		updateMenus();
	}

	public boolean menuItemState(int command) {
		AlexsaWindow fw = getFrontMostWindow();
		switch (command) {
			case cmd_Close:
			case cmd_ViewSets:
			case cmd_ViewDeclarations:
			case cmd_ViewCorePredicates:
			case cmd_ViewInstrumentationPredicates:
			case cmd_ViewConsistencyRules:
			case cmd_ViewActions:
			case cmd_ExportMenu:
			case cmd_ExportCodeView:
			case cmd_ExportTraversion:
			case cmd_up:
			case cmd_down:
			case cmd_forward:
			case cmd_backward:
				return fw != null;
		}
		return true;
	}

	public String menuItemName(int cmd, String old_name) {
		return old_name;
	}


	public void actionPerformed(ActionEvent e) { }

}
