package alexsa;

import java.io.*;
import alexsa.data.animation.Analysis;
import alexsa.gui.dialogs.*;

public class Preferences {
	public final static String
		prefs_filename = "Alexsa.prefs";
		
	private final static String
		EXTERNAL_LABEL	= "EXTERNAL_CYCLES",
		INTERNAL_LABEL	= "INTERNAL_CYCLES",
		BRANCHED_LABEL	= "BRANCHED",
		STRAIGHT_LABEL	= "STRAIGHT",
		FORWARD_LABEL	= "FORWARD_EDGES",
		CROSS_LABEL		= "CROSS_EDGES";

	// How many times should we execute a cycle before stopping
	// with it?
	public static int
		number_of_cycles = 3;

	// The delay between standard-animation steps
	public static int
		speed = 2000;
	
	public static String
		startup_path = "./",
		code_view_indention = "    ";
	
	public static boolean
		summarize_unknown_branches = true;
		
	// the following variables define the timing for the
	// inbetweening of shape graphs
	public static int
		MASTER_SPEED = 10,

		ZOOM_STEPS = 5,
		ZOOM_DELAY = 10,

		REMOVE_STEPS = 10,
		REMOVE_DELAY = 5,

		CREATE_STEPS = 10,
		CREATE_DELAY = 5,

		MOVE_STEPS = 10,
		MOVE_DELAY = 8,

		MERGE_STEPS = 10,
		MERGE_DELAY = 8,

		SPLIT_STEPS = 10,
		SPLIT_DELAY = 8,
		
		FINAL_DELAY = 100;

	// this is the order in which branches are chosen
	public static int[]
		order = {
			Analysis.EXTERNAL_CYCLE,
			Analysis.INTERNAL_CYCLE,
			Analysis.BRANCHED,
			Analysis.STRAIGHT,
			Analysis.FORWARD_EDGING,
			Analysis.CROSS_EDGING
		};
	
	private static void println(BufferedWriter bw, String s)
		throws IOException
	{
		print(bw, s);
		bw.write('\n');
	}
	
	private static void print(BufferedWriter bw, String s)
		throws IOException
	{
		bw.write(s, 0, s.length());
	}
	
	public static void savePrefs() {
		File f = new File(prefs_filename);
		try {
			BufferedWriter bw = new BufferedWriter(new FileWriter(f));
			println(bw, 
				"# Preferences for Alexsa - Algorithm Explanation by Shape Analysis\n" +
				"# This file allows for the specification of new default values.\n" +
				"# Lines starting with a '#' are comments and are fully ignored.\n" +
				"\n" +
				"# Warning: This file will be overwritten when selecting \"Save Prefs\"\n" +
				"# from the Options-menu in Alexsa!\n" +
				"\n" +
				"# All entries have the form\n" +
				"# variable = value\n" +
				"# The variable's name is treated case-insensitive\n" +
				"# The value may be quoted with \"-quotes\n");

			println(bw,
				"# The directory that is presented to the user when opening a new\n" +
				"# TVP file\n" +
				"# Default:\n" +
				"# startup_path = \"./\"");
			if (!startup_path.equals("./"))
				println(bw, "startup_path = \"" + startup_path + "\"\n");

			println(bw,
				"# The number of times a cycle is executed before leaving it\n" +
				"# when animating the program\n" +
				"# Default:\n" +
				"# number_of_cycles = 3");
			if (number_of_cycles != 3)
				println(bw, "number_of_cycles = " + number_of_cycles + "\n");

			println(bw,
				"# In the code view, nested constructions are indented for clarity.\n" +
				"# Default:\n" +
				"# code_view_indention = \"    \"\n");
			if (!code_view_indention.equals("    "))
				println(bw, "code_view_indention = \"" + code_view_indention + "\"\n");
			
			println(bw,
				"# When constructing the code view, Alexsa can summarize unmappable\n" +
				"# comparisons (like value-comparisons)\n" +
				"# Default:\n" +
				"# summarize_unknown_branches = true\n");
			if (!summarize_unknown_branches)
				println(bw, "summarize_unknown_branches = false\n");

			println(bw,
				"# We can control the timing of the inbetweening by changing these\n" +
				"# values. All animation execute a fixed number of steps, and no step\n" +
				"# will be faster than some given minimum time. The absolut time in\n" +
				"# milliseconds results from the product of the master speed and the\n" +
				"# delay for some operation. Smaller values for master speed and specific\n" +
				"# delays result in faster animations.\n" +
				"\n" +
				"# Default values as follows:\n" +
				"# MASTER_SPEED = 10\n" +
				"# ZOOM_STEPS = 5\n" +
				"# ZOOM_DELAY = 10\n" +
				"# REMOVE_STEPS = 10\n" +
				"# REMOVE_DELAY = 5\n" +
				"# CREATE_STEPS = 10\n" +
				"# CREATE_DELAY = 5\n" +
				"# MOVE_STEPS = 10\n" +
				"# MOVE_DELAY = 8\n" +
				"# MERGE_STEPS = 10\n" +
				"# MERGE_DELAY = 8\n" +
				"# SPLIT_STEPS = 10\n" +
				"# SPLIT_DELAY = 8\n" +
				"# FINAL_DELAY = 100\n");
			if (MASTER_SPEED != 10)
				println(bw, "MASTER_SPEED = " + MASTER_SPEED);
			if (ZOOM_STEPS != 5)
				println(bw, "ZOOM_STEPS = " + ZOOM_STEPS);
			if (ZOOM_DELAY != 10)
				println(bw, "ZOOM_DELAY = " + ZOOM_DELAY);
			if (REMOVE_STEPS != 10)
				println(bw, "REMOVE_STEPS = " + REMOVE_STEPS);
			if (REMOVE_DELAY != 5)
				println(bw, "REMOVE_DELAY = " + REMOVE_DELAY);
			if (CREATE_STEPS != 10)
				println(bw, "CREATE_STEPS = " + CREATE_STEPS);
			if (CREATE_DELAY != 5)
				println(bw, "CREATE_DELAY = " + CREATE_DELAY);
			if (MOVE_STEPS != 10)
				println(bw, "MOVE_STEPS = " + MOVE_STEPS);
			if (MOVE_DELAY != 8)
				println(bw, "MOVE_DELAY = " + MOVE_DELAY);
			if (MERGE_STEPS != 10)
				println(bw, "MERGE_STEPS = " + MERGE_STEPS);
			if (MERGE_DELAY != 8)
				println(bw, "MERGE_DELAY = " + MERGE_DELAY);
			if (SPLIT_STEPS != 10)
				println(bw, "SPLIT_STEPS = " + SPLIT_STEPS);
			if (SPLIT_DELAY != 8)
				println(bw, "SPLIT_DELAY = " + SPLIT_DELAY);
			if (FINAL_DELAY != 100)
				println(bw, "FINAL_DELAY = " + FINAL_DELAY);

			println(bw,
				"# To compute the linearization of shape graphs for visual\n" +
				"# execution, Alexsa uses a set of priorities for different\n" +
				"# phenomena. The order in which these phenomena are handled\n" +
				"# decides on the actual order of execution.\n" +
				"\n" +
				"# Default (make sure to use *exactly* these strings):\n" +
				"# order = " +
				EXTERNAL_LABEL + ", " +
				INTERNAL_LABEL + ", " +
				BRANCHED_LABEL + ", " +
				STRAIGHT_LABEL + ", " +
				FORWARD_LABEL + ", " +
				CROSS_LABEL);
			if (order[0] != Analysis.EXTERNAL_CYCLE ||
				order[1] != Analysis.INTERNAL_CYCLE ||
				order[2] != Analysis.BRANCHED ||
				order[3] != Analysis.STRAIGHT ||
				order[4] != Analysis.FORWARD_EDGING ||
				order[5] != Analysis.CROSS_EDGING)
			{
				print(bw, "order = ");
				for (int i = 0; i < 6; i++) {
					switch (order[i]) {
						case Analysis.STRAIGHT:
							print(bw, STRAIGHT_LABEL);
							break;
						case Analysis.INTERNAL_CYCLE:
							print(bw, INTERNAL_LABEL);
							break;
						case Analysis.EXTERNAL_CYCLE:
							print(bw, EXTERNAL_LABEL);
							break;
						case Analysis.BRANCHED:
							print(bw, BRANCHED_LABEL);
							break;
						case Analysis.CROSS_EDGING:
							print(bw, CROSS_LABEL);
							break;
						case Analysis.FORWARD_EDGING:
							print(bw, FORWARD_LABEL);
							break;
					}
					if (i < 5)
						bw.write(", ", 0, 2);
				}
				bw.write('\n');
			}
			bw.close();
		} catch (IOException e) {
			System.err.println("The preferences were not written correctly due to an I/O-error.");
		}
	}

	public static void readPrefs() {
		File f = new File (prefs_filename);
		try {
			BufferedReader br = new BufferedReader(new FileReader(f));
			String line = null;
			int line_nr = 0;
			while ((line = br.readLine()) != null) {
				line = line.trim();
				line_nr++;
				// skip comment lines
				if (line.startsWith("#") || line.length() == 0)
					continue;
				int equals = line.indexOf('=');
				if (equals <= 0) {
					System.err.println("Syntax error in preferences file \"" +
						prefs_filename + "\" in line " + line_nr + ": No '=' found:");
					System.err.println(line);
					continue;
				}
				String variable = line.substring(0, equals).trim().toLowerCase();
				if (variable.length() == 0) {
					System.err.println("Syntax error in preferences file \"" +
						prefs_filename + "\" in line " + line_nr + ": No variable name found:");
					System.err.println(line);
					continue;
				}
				String value = line.substring(equals + 1).trim();
				if (value.length() >= 2 && value.startsWith("\"") && value.endsWith("\""))
					value = value.substring(1, value.length() - 1);
				if (variable.equals("startup_path")) {
					startup_path = value;
				} else if (variable.equals("number_of_cycles")) {
					number_of_cycles = Integer.parseInt(value);
				} else if (variable.equals("code_view_indention")) {
					code_view_indention = value;
				} else if (variable.equals("summarize_unknown_branches")) {
					if (value.equalsIgnoreCase("true"))
						summarize_unknown_branches = true;
					else if (value.equalsIgnoreCase("false"))
						summarize_unknown_branches = false;
					else {
						System.err.println("Syntax error in preferences file \"" +
							prefs_filename + "\" in line " + line_nr + ": Expected value \"true\" or \"false\"");
						System.err.println(line);
						continue;
					}
				} else if (variable.equals("master_speed")) {
					MASTER_SPEED = Integer.parseInt(value);
				} else if (variable.equals("zoom_steps")) {
					ZOOM_STEPS = Integer.parseInt(value);
				} else if (variable.equals("zoom_delay")) {
					ZOOM_DELAY = Integer.parseInt(value);
				} else if (variable.equals("remove_steps")) {
					REMOVE_STEPS = Integer.parseInt(value);
				} else if (variable.equals("remove_delay")) {
					REMOVE_DELAY = Integer.parseInt(value);
				} else if (variable.equals("create_steps")) {
					CREATE_STEPS = Integer.parseInt(value);
				} else if (variable.equals("create_delay")) {
					CREATE_DELAY = Integer.parseInt(value);
				} else if (variable.equals("move_steps")) {
					MOVE_STEPS = Integer.parseInt(value);
				} else if (variable.equals("move_delay")) {
					MOVE_DELAY = Integer.parseInt(value);
				} else if (variable.equals("merge_steps")) {
					MERGE_STEPS = Integer.parseInt(value);
				} else if (variable.equals("merge_delay")) {
					MERGE_DELAY = Integer.parseInt(value);
				} else if (variable.equals("split_steps")) {
					SPLIT_STEPS = Integer.parseInt(value);
				} else if (variable.equals("split_delay")) {
					SPLIT_DELAY = Integer.parseInt(value);
				} else if (variable.equals("final_delay")) {
					FINAL_DELAY = Integer.parseInt(value);
				} else if (variable.equals("order")) {
					int i = 0;
					while (value.length() > 0 && i < 6) {
						String s;
						int pos = value.indexOf(',');
						if (pos >= 0) {
							s = value.substring(0, pos).trim();
							value = value.substring(pos + 1);
						} else {
							s = value.trim();
							value = "";
						}
						if (s.equals(EXTERNAL_LABEL)) {
							order[i] = Analysis.EXTERNAL_CYCLE;
						} else if (s.equals(INTERNAL_LABEL)) {
							order[i] = Analysis.INTERNAL_CYCLE;
						} else if (s.equals(STRAIGHT_LABEL)) {
							order[i] = Analysis.STRAIGHT;
						} else if (s.equals(BRANCHED_LABEL)) {
							order[i] = Analysis.BRANCHED;
						} else if (s.equals(CROSS_LABEL)) {
							order[i] = Analysis.CROSS_EDGING;
						} else if (s.equals(FORWARD_LABEL)) {
							order[i] = Analysis.FORWARD_EDGING;
						}
						i++;
					}
				} else
					System.err.println("Unknown variable: [" + variable + "] in line " + line_nr);
			}
			br.close();
		} catch (IOException e) {
			System.err.println("The preferences were not read correctly due to an I/O-error.");
		}
	}

	public static void changeMiscPrefs() {
		new CodeViewPrefsDialog();
	}
	public static void changeAnimationPrefs() {
		new AnimationPrefsDialog();
	}
}
