package alexsa.data;

import java.util.Vector;

public class Node {
	
	public final static int
		WHITE	= 0,	// node untouched during DFS
		GRAY	= 1,	// node being examined during DFS
		BLACK	= 2;	// node has been fully examined

	public int
		color	= WHITE,
		d		= 0,			// discovery time for DFS
		f		= 0;			// finishing time for DFS

	protected Vector
		out	= null,
		in	= null;

	// Even though a node can have multiple parents, this
	// variable will specify the ancestor during a DFS starting
	// at the initial shapes
	public Node
		parent = null;

	public String
		name = null;


	public int outDegree() {
		return out != null ? out.size() : 0;
	}

	public int inDegree() {
		return in != null ? in.size() : 0;
	}
	
	public void addInEdge(Edge t) {
		if (in == null)
			in = new Vector();
		in.addElement(t);
	}
	
	public void setInEdges(Vector v) {
		in = v;
	}

	public void addOutEdge(Edge t) {
		if (out == null)
			out = new Vector();
		out.addElement(t);
	}

	public void setOutEdges(Vector v) {
		out = v;
	}

	public Edge getInEdge(int i) {
		if (i >= 0 && i < inDegree())
			return (Edge)in.elementAt(i);
		else
			return null;
	}
	
	public boolean removeInEdge(Edge e) {
		for (int i = inDegree() - 1; i >= 0; i--)
			if (getInEdge(i) == e) {
				in.removeElementAt(i);
				return true;
			}
		return false;
	}
	
	public boolean removeOutEdge(Edge e) {
		for (int i = outDegree() - 1; i >= 0; i--)
			if (getOutEdge(i) == e) {
				out.removeElementAt(i);
				return true;
			}
		return false;
	}
	
	public Vector getInEdges() {
		return in;
	}

	public Edge getOutEdge(int i) {
		if (i >= 0 && i < outDegree())
			return (Edge)out.elementAt(i);
		else
			return null;
	}
	
	public Edge getOutEdgeTo(Node to) {
		for (int i = 0; i < outDegree(); i++)
			if (getOutEdge(i).target == to)
				return (Edge)getOutEdge(i);
		return null;
	}

	public Vector getOutEdges() {
		return out;
	}

	protected static int time;
	public static void dfs(Node u) {
		time = 1;
		dfs_visit(u);
	}
	
	public static void dfs(Node[] nodes) {
		time = 1;
		for (int i = 0; i < nodes.length; i++)
			if (nodes[i].color == WHITE)
				dfs_visit(nodes[i]);
	}
	
	public static void dfs_visit(Node u) {
		u.color = GRAY;
		u.d = time++;
		for (int i = 0; i < u.outDegree(); i++) {
			Edge v = u.getOutEdge(i);
			if (v.target == null)
				v.type = Edge.NO_EDGE;
			else
				switch (v.target.color) {
					case WHITE:
						v.type = Edge.TREE_EDGE;
						v.target.parent = u;
						dfs_visit(v.target);
						break;
					case GRAY:
						v.type = Edge.BACK_EDGE;
						break;
					case BLACK:
						if (u.d < v.target.d)
							v.type = Edge.FORWARD_EDGE;
						else
							v.type = Edge.CROSS_EDGE;
						break;
				}
		}
		u.color = BLACK;
		u.f = time++;
	}

	
	public boolean find(Node n) {
		if (this == n)
			return true;
		for (int i = outDegree() - 1; i >= 0; i--) {
			Edge e = getOutEdge(i);
			if (e.type == Edge.TREE_EDGE || e.type == Edge.CROSS_EDGE) {
				if (e.target.find(n))
					return true;
			}
		}
		return false;
	}

	public void replaceInOutEdges(Node old_node, Node new_node) {
		for (int i = outDegree() - 1; i >= 0; i--) {
			Edge e = getOutEdge(i);
			if (e.target == old_node)
				e.target = new_node;
		}
	}

	public void replaceInInEdges(Node old_node, Node new_node) {
		for (int i = inDegree() - 1; i >= 0; i--) {
			Edge e = getInEdge(i);
			if (e.source == old_node)
				e.source = new_node;
		}
	}

	public String toString() {
		return name;
	}
}
