package alexsa.data.animation;

import java.util.Enumeration;
import java.util.Vector;

import alexsa.Preferences;
import alexsa.data.Edge;
import alexsa.data.Node;
import alexsa.data.tvp.CfgNode;
import alexsa.data.graphs.ShapeGraph;

public class Animation {
	private Vector
		list = null,
		stack = null;

	private int
		current_shape = -1;

	private boolean
		jump = true;

	public Animation(ShapeGraph shape, Trace t) {
		stack = new Vector();
		stack.addElement(shape);
		
		list = new Vector();
		list.addElement(new Step(shape, true));
		
		current_shape = 0;
		
		// reset all passes-counters from previous animations
		Enumeration e = t.transitions();
		while (e.hasMoreElements())
			((Transition)e.nextElement()).passes = 0;
		
		ShapeGraph next = getNextShape();
		while (next != null) {
			list.addElement(new Step(next, jump));
			next = getNextShape();
		}
	}
	
	public Step getInitialShape() {
		if (list != null && list.size() > 0)
			return (Step)list.firstElement();
		else
			return null;
	}
	
	public Step getPreviousStep() {
		if (current_shape > 0)
			return (Step)list.elementAt(--current_shape);
		return null;
	}

	public Step getCurrentStep() {
		if (current_shape >= 0)
			return (Step)list.elementAt(current_shape);
		else
			return null;
	}

	public ShapeGraph getCurrentShapeGraph() {
		if (current_shape >= 0)
			return ((Step)list.elementAt(current_shape)).shape_graph;
		else
			return null;
	}

	public Step getNextStep() {
		if (current_shape < list.size() - 1)
			return (Step)list.elementAt(++current_shape);
		else
			return null;
	}
	
	public Vector getAllSteps() {
		return list;
	}
	
	// set the current_shape pointer back to the first shape graph
	// in this animation
	public void resetAnimation() {
		current_shape = 0;
	}
	
	public void setAnimationTo(int i) {
		current_shape = i;
	}
	
	public int getFirstStepForCfgNode(CfgNode cn) {
		for (int i = 0; i < list.size(); i++) {
			Step step = (Step)list.elementAt(i);
			for (int j = 0; j < cn.shape_graphs.size(); j++)
				if (cn.shape_graphs.elementAt(j) == step.shape_graph)
					return i;
		}
		return -1;
	}
	
	public int getFirstStepForShapeGraph(ShapeGraph sg) {
		for (int i = 0; i < list.size(); i++) {
			Step step = (Step)list.elementAt(i);
			if (step.shape_graph == sg)
				return i;
		}
		return -1;
	}
	
	private boolean hasValidStepExcept(Transition t, ShapeGraph sg) {
		boolean has_it = false;
		Transition t2 = null;
		for (int i = sg.outDegree() - 1; !has_it && i >= 0; i--)
			if ((t2 = (Transition)sg.getOutEdge(i)).valid() && t2 != t)
				has_it = true;
		return has_it;
	}

	private Transition getStepFrom(ShapeGraph g) {
		boolean Debug = false;
		Vector options = null;
		if (g.outDegree() > 0) {
			for (int i = 0; i < g.outDegree(); i++) {
				Transition t = (Transition)g.getOutEdge(i);
				if (t.valid()) {
					if (options == null)
						options = new Vector();
					options.addElement(t);
					if (Debug)
						System.out.println("One valid option: " + t);
				} else if (Debug)
					System.out.println("Invalid option: " + t);
			}
		}
		if (options == null)
			return null;
		if (options.size() == 1)
			return (Transition)options.elementAt(0);
		
		for (int i = 0; i < options.size(); i++) {
			Transition t2 = (Transition)options.elementAt(i);
			t2.analysis = new Analysis(t2);
		}

		// Now: narrow down selection to roughly equivalent transitions,
		// that is, those that have the same level of complexity
		int i = 0;
		Transition t = null;
		while (t == null && i < Preferences.order.length && options.size() > 1) {
			int criteria = Preferences.order[i];
			if (someMatchCriteria(criteria, options)) {
				if (Debug)
					System.out.println("some match criteria " + criteria);
				if (allMatchCriteria(criteria, options)) {
					if (Debug)
						System.out.println("all match criteria " + criteria);
					t = singleBestTransition(criteria, options);
					if (Debug && t != null)
						System.out.println("single best: " + t);
				} else {
					if (Debug)
						System.out.println("some don't match criteria " + criteria);
					removeNonMatchingTransitions(criteria, options);
					t = singleBestTransition(criteria, options);
				}
			}
			i++;
		}
		if (t == null && options.size() == 1) {
			t = (Transition)options.elementAt(0);
			if (Debug)
				System.out.println("just one left: " + t);
		}

		if (t == null && options.size() > 1) {
			// random decision
			i = (int)Math.floor(Math.random() * options.size());
			t = (Transition)options.elementAt(i);
			if (Debug)
				System.out.println("randomly selected " + t + " out of " + options.size());
		}
		return t;
	}

	private ShapeGraph getNextShape() {
		int i = stack.size() - 1;
		Transition t = null;
		
		while (i >= 0 && (t = getStepFrom((ShapeGraph)stack.elementAt(i))) == null)
			i--;

		if (t != null && i != (stack.size() - 1)) {
			stack.setSize(i + 1);
			jump = true;
		} else
			jump = false;

		if (t != null) {
			// use it
			
			// for back_edges, we reset the passes-counter of every edge
			// between the target and the source of this node.
			if (t.type == Edge.BACK_EDGE) {
				int loop_start = stack.indexOf(t.target);
				
				if (loop_start >= 0) {
					int loop_end = i;
					// special treatment for the last cycle: if possible,
					// find an unful branching and reset the passes-variables
					// only up to there.
					if (t.passes == Preferences.number_of_cycles - 1) {
						while (loop_end >= 0 &&
							!hasValidStepExcept(t, (ShapeGraph)stack.elementAt(loop_end)))
						{
							loop_end--;
						}
					}

					for (int j = loop_start; j < loop_end; j++) {
						Node
							source = (Node)stack.elementAt(j),
							target = (Node)stack.elementAt(j + 1);
						Transition t2 = (Transition)source.getOutEdgeTo(target);
						t2.passes = 0;
					}
					stack.setSize(loop_start);
				}
			}
			stack.addElement(t.target);
			t.passes++;
			return (ShapeGraph)t.target;
		} else
			return null;
	}
		
	protected Transition singleBestTransition(int criteria, Vector v) {
		int best_value = 0;
		Transition best = null;
		boolean multiple = true;
		for (int i = 0; i < v.size(); i++) {
			Transition t = (Transition)v.elementAt(i);
			int quality = t.analysis.quality(criteria);
			if (quality == best_value)
				multiple = true;
			else if (quality > best_value) {
				multiple = false;
				best_value = quality;
				best = t;
			}
		}
		return multiple ? null : best;
	}
	
	protected void removeNonMatchingTransitions(int criteria, Vector v) {
		int i = 0;
		while (i < v.size()) {
			Transition t = (Transition)v.elementAt(i);
			if (!t.analysis.matches(criteria))
				v.removeElementAt(i);
			else
				i++;
		}
	}
	
	protected boolean someMatchCriteria(int criteria, Vector v) {
		for (int i = 0; i < v.size(); i++) {
			Transition t = (Transition)v.elementAt(i);
			if (t.analysis.matches(criteria))
				return true;
		}
		return false;
	}
	
	protected boolean allMatchCriteria(int criteria, Vector v) {
		for (int i = 0; i < v.size(); i++) {
			Transition t = (Transition)v.elementAt(i);
			if (!t.analysis.matches(criteria))
				return false;
		}
		return true;
	}

}
