package alexsa.data.animation;

import java.io.*;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import alexsa.data.tvp.Program;
import alexsa.data.graphs.ShapeGraph;
import alexsa.data.morphing.Morphing;
import alexsa.data.*;
import alexsa.Preferences;

public class Trace {

	public ShapeGraph[]
		initial_shapes = null;
	
	protected Vector
		cycles = null;

	public Vector
		locations = null;

	public Animation[]
		animations = null;
	
	public Hashtable
		shapes = new Hashtable();

	public Trace(File f, Vector graphs, Program program)
		throws FileFormatException, IOException
	{
		BufferedReader
			in = new BufferedReader(new FileReader(f));
		String
			line = null,
			first_location = null;
		int
			graph_index = 0;
		ShapeGraph
			g = null;
		Vector
			initial_shapes_indexes = new Vector(4),
			current_location = null;
		Hashtable
			out_edges = new Hashtable();
		locations = new Vector();

		if (((ShapeGraph)graphs.elementAt(0)).getNumberOfNodes() > 1)
			program.cfg = (ShapeGraph)graphs.elementAt(0);
		
		while ((line = in.readLine()) != null) {
			if (line.startsWith("initial: "))
				initial_shapes_indexes.add(new Integer(line.substring(line.indexOf(' ') + 1)));
			else if (	line.startsWith("transition: ") || 
						line.startsWith("redundancy: "))
				putRawTransition(out_edges, line);
			else if (line.startsWith("location: ")) {
				current_location = new Vector();
				current_location.addElement(line.substring(line.indexOf(' ') + 1));
				locations.addElement(current_location);
				if (first_location == null) {
					first_location = line.substring(line.indexOf(' ') + 1);
					boolean found_first = false;
					while (!found_first) {
						g = (ShapeGraph)graphs.elementAt(graph_index++);
						if (g.getNumberOfNodes() == 1 &&
							g.getNode(0).name.equals(first_location))
								found_first = true;
					}
				} else
					graph_index++;
			} else if (line.startsWith("messages for: "))
				graph_index++;
			else if (line.startsWith("shape: ")) {
				g = (ShapeGraph)graphs.elementAt(graph_index++);
				Integer I = new Integer(line.substring(line.indexOf(' ') + 1));
				g.identifier = I.intValue();
				shapes.put(I, g);
				current_location.addElement(g);
			} else if (line.startsWith("message: "))
				graph_index++;
			else
				throw new FileFormatException(
					"illegal command in trace file: [" + line +"]");
		}
		in.close();

		// we can now throw away all unneccessary graphs
		graphs.removeAllElements();

		Enumeration e = shapes.keys();
		while (e.hasMoreElements()) {
			Integer I = (Integer)e.nextElement();
			ShapeGraph s = (ShapeGraph)shapes.get(I);

			// get all transitions leaving s
			Object o = out_edges.get(I);
			if (o instanceof Vector) {
				Vector v = (Vector)o;
				for (int i = 0; i < v.size(); i++) {
					ShapeGraph target = (ShapeGraph)shapes.get(v.elementAt(i));
					Transition t = new Transition(s, target);
					s.addOutEdge(t);
					if (target != null)
						target.addInEdge(t);
				}
			} else if (o instanceof Integer) {
				ShapeGraph target = (ShapeGraph)shapes.get(o);
				Transition t = new Transition(s, target);
				s.addOutEdge(t);
				if (target != null)
					target.addInEdge(t);
			}
		}

		// create array of initial shapes
		initial_shapes = new ShapeGraph[initial_shapes_indexes.size()];
		for (int i = 0; i < initial_shapes_indexes.size(); i++) {
			Integer I = (Integer)initial_shapes_indexes.elementAt(i);
			initial_shapes[i] = (ShapeGraph)shapes.get(I);
		}
		initial_shapes_indexes.removeAllElements();
		
		classifyEdges(shapes);
		
//		determineMaxPasses();
	}
	
	protected void classifyEdges(Hashtable shapes) {
		Node.dfs(initial_shapes);

		cycles = new Vector();
		Enumeration e = shapes.elements();
		while (e.hasMoreElements()) {
			Node n = (Node)e.nextElement();
			for (int i = n.outDegree() - 1; i >= 0; i--) {
				Edge edge = n.getOutEdge(i);
				if (edge.type == Edge.BACK_EDGE)
					cycles.addElement(edge);
			}
		}
	}
		
	/*	for inner cycles, the maximum number of passes is the product
		of its usual maximum number of cycles and the number of cycles
		of its enclosing cycle.
	*/
	// XXX Check: order of execution relevant?
/*	protected void determineMaxPasses() {
		Vector
			starting_circles = new Vector(),
			inner_circles = new Vector();
		Transition t, t2;
		Node s;
		for (int i = 0; i < cycles.size(); i++) {
			t = (Transition)cycles.elementAt(i);
			inner_circles.removeAllElements();
			starting_circles.removeAllElements();
			s = t.source;
			while (s != t.target && s != null) {
				for (int j = 0; j < s.outDegree(); j++)
					if ((t2 = (Transition)s.getOutEdge(j)).type == Transition.BACK_EDGE && t2 != t)
						starting_circles.addElement(t2);
				s = s.parent;
				for (int j = 0; j < starting_circles.size(); j++)
					if (((Transition)starting_circles.elementAt(j)).target == s) {
						inner_circles.addElement(starting_circles.elementAt(j));
						starting_circles.removeElementAt(j);
					}
			}
			for (int j = 0; j < inner_circles.size(); j++)
				((Transition)inner_circles.elementAt(j)).max_passes =
					t.max_passes * Preferences.number_of_cycles +
						Preferences.number_of_cycles;
		}
/*		for (int i = 0; i < cycles.size(); i++) {
			t = (Transition)cycles.elementAt(i);
			System.out.println("Max Passes for " + t.source + " -> " + t.target + " = " + t.max_passes);
		}*/
//	}

	protected void putRawTransition(Hashtable out_edges, String s)
		throws FileFormatException
	{
		String[] parts = Tools.split(s, ' ');
		if (parts.length != 8)
			throw new FileFormatException(
				"8 tokens expected in line [" +
				s + "], but " + parts.length + " found.");

		checkSyntax(parts[2], "->", s);
		checkSyntax(parts[6], "->", s);

		Integer
			shape_a = new Integer(parts[5]),
			shape_b = new Integer(parts[7]);
		
		Object o = out_edges.get(shape_a);
		if (o == null)
			out_edges.put(shape_a, shape_b);
		else {
			if (o instanceof Vector)
				((Vector)o).addElement(shape_b);
			else if (o instanceof Integer) {
				Vector v = new Vector();
				v.addElement(o);
				v.addElement(shape_b);
				out_edges.put(shape_a, v);
			}
		}
	}
	
	protected void checkSyntax(String is, String shouldBe, String wholeLine)
		throws FileFormatException
	{
		if (!is.equals(shouldBe))
			throw new FileFormatException(
				"[" + shouldBe + "] expected in line [" +
				wholeLine + "], but [" + is + "] found.");
	}
	
	public ShapeGraph getShapeGraph(int i) {
		return (ShapeGraph)shapes.get(new Integer(i));
	}
	
	/* All animations (instances of alexsa.data.animation.Animation) are
	   fully precomputed. This will be useful for walking step-by-step
	   through the animation.
	   There will be one animation for each initial shape.
	*/
	public void createAnimations() {
		animations = new Animation[initial_shapes.length];
		for (int i = 0; i < initial_shapes.length; i++)
			animations[i] = new Animation(initial_shapes[i], this);
	}

	public void exportAnimations(BufferedWriter bw)
		throws IOException
	{
		for (int i = 0; i < animations.length; i++) {
			Vector v = animations[i].getAllSteps();
			bw.write("Animation [" + (i + 1) + "/" + animations.length + "]\n");
			for (int j = 0; j < v.size(); j++) {
				Step step = (Step)v.elementAt(j);
				bw.write(step.shape_graph.identifier + ", ");
				if (step.jump)
					bw.write("jump\n");
				else
					bw.write("consecutive\n");
			}
		}
	}

	public Enumeration shapes() {
		return new ShapeEnumeration();
	}
	
	class ShapeEnumeration implements Enumeration {
		private
			int loc_id = 0,
			shape_id = 1;

		public boolean hasMoreElements() {
			return loc_id < locations.size() &&
				shape_id < ((Vector)locations.elementAt(loc_id)).size();
		}

		public Object nextElement() {
			if (!hasMoreElements())
				return null;
			Vector v = (Vector)locations.elementAt(loc_id);
			Object o = v.elementAt(shape_id);
			shape_id++;
			if (shape_id >= v.size()) {
				loc_id++;
				shape_id = 1;
			}
			return o;
		}
	}

	public Enumeration transitions() {
		return new TransitionEnumeration();
	}
	
	class TransitionEnumeration implements Enumeration {
		private int
			i = 0;
		private Enumeration
			shapes = null;
		private ShapeGraph
			shape = null;
			
		public TransitionEnumeration() {
			shapes = new ShapeEnumeration();
			shape = (ShapeGraph)shapes.nextElement();
		}

		public boolean hasMoreElements() {
			return shape != null && i < shape.outDegree();
		}

		public Object nextElement() {
			if (!hasMoreElements())
				return null;
			Object o = shape.getOutEdge(i);
			i++;
			if (i >= shape.outDegree()) {
				shape = (ShapeGraph)shapes.nextElement();
				i = 0;
			}
			return o;
		}
	}
}
