package alexsa.data.graphs;

import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.Point;
import java.util.Vector;

import alexsa.data.Edge;

public class GraphEdge extends Edge implements Cloneable {
	public String
		source_name = null,
		target_name = null;

	public Vector
		points = null;
		
	public int
		n = 0;

	public EdgeLabel
		label = null;

	public Attributes
		attributes = null;

	public GraphEdge(String tailname, String headname, int i, Vector p, EdgeLabel dl, Attributes a)
	{
		source_name = tailname;
		target_name = headname;
		n = i;
		points = p;
		label = dl;
		attributes = a;
	}

	public String getLabel() {
		return label == null ? null : label.label;
	}
	
	public boolean isDotted() {
		return attributes != null && attributes.style == GraphSymbols.DOTTED;
	}
	
	public Object clone() {
		Vector v = new Vector(points.size());
		for (int i = 0; i < points.size(); i++)
			v.addElement(((Point)points.elementAt(i)).clone());

		GraphEdge ge = new GraphEdge(source_name, target_name, n, v,
			null, null);
		if (label != null)
			ge.label = (EdgeLabel)label.clone();
		if (attributes != null)
			ge.attributes = (Attributes)attributes.clone();
		
		ge.type = type;
		return ge;
	}

	// are the given coordinates within the area occupied by this
	// edge's label? If there is no label, contains returns false
	// Since we need to calc the label's width and height, we also
	// need some FontMetrics. For efficiency, these are passed in
	// as a parameter, so they don't need to be recalculated
	// every time.
	public boolean contains(Point p, FontMetrics fm) {
		if (label == null || label.location == null)
			return false;
		return
			(Math.abs(p.x - label.location.x) <= fm.stringWidth(label.label) / 2) &&
			(Math.abs(p.y - label.location.y) <= fm.getHeight());
	}

	public String toString() {
		if (label != null)
			return "GraphEdge from " + source_name +
				" to " + target_name + ", edge-label " + label.label;
		else
			return "GraphEdge from " + source_name +
				" to " + target_name + ", no edge-label";
	}
}
