package alexsa.data.morphing;

import java.util.Vector;

import alexsa.data.graphs.Attributes;
import alexsa.data.graphs.EdgeLabel;
import alexsa.data.graphs.GraphNode;
import alexsa.data.graphs.GraphEdge;
import alexsa.data.Edge;

/* this class is a common ancestor for Split and Merge.
	It contains the appropriate variables and methods
	needed by both to handle the change to multiple
	nodes at once
*/

public class MultiMorph extends AbstractMorph {
	public GraphNode
		sources[] = null,	// sources will receive two copies of
							// the source node, which will then be
							// moved to the respective target-positions
		targets[] = null;	// targets contains the two new nodes
	public Vector
		new_edges,
		old_edges,		// edges disappearing during the morph
		internal_edges; // edges between the two morphed nodes nodes
	
	public MultiMorph(GraphNode s) {
		super(s);
	}

	public MultiMorph(GraphNode s, GraphNode t) {
		super(s, t);
	}

	public void init() {
	}
	
	/* Does the given node has a self loop with a label identical to that
		of the specified edge e?
	*/
	protected boolean hasSelfLoopLikeThis(GraphNode n, GraphEdge e) {
		for (int i = 0; i < n.inDegree(); i++) {
			GraphEdge loop = (GraphEdge)n.getInEdge(i);
			if (loop.source == n && loop.label.label.equals(e.label.label))
				return true;
		}
		return false;
	}
	
	protected GraphEdge getEdge_SourceSignificant(GraphNode source, GraphNode target, EdgeLabel label) {
		if (source.significant_label == null)
			return null;
		for (int i = 0; i < target.inDegree(); i++) {
			GraphEdge e = (GraphEdge)target.getInEdge(i);
			GraphNode possible_source = (GraphNode)e.source;
			if (possible_source.significant_label != null &&
				possible_source.significant_label.equals(source.significant_label) &&
				((label == null && e.label == null) ||
				(label != null && e.label != null && label.label.equals(e.label.label))))
					return e;
		}
		return null;
	}

	/* return an edge leading from source to target, based on target's significant
		label.
		If no appropriate edge was found, return null
	*/
	protected GraphEdge getEdge_TargetSignificant(GraphNode source, GraphNode target, EdgeLabel label) {
		if (target.significant_label == null)
			return null;
		for (int i = 0; i < source.outDegree(); i++) {
			GraphEdge e = (GraphEdge)source.getOutEdge(i);
			GraphNode possible_target = (GraphNode)e.target;
			if (possible_target.significant_label != null &&
				possible_target.significant_label.equals(target.significant_label) &&
				((label == null && e.label == null) ||
				(label != null && e.label != null && label.label.equals(e.label.label))))
					return e;
		}
		return null;
	}
}