package alexsa.data.morphing;

import java.util.Vector;

import alexsa.data.graphs.Attributes;
import alexsa.data.graphs.EdgeLabel;
import alexsa.data.graphs.GraphNode;
import alexsa.data.graphs.GraphEdge;
import alexsa.data.Edge;

public class Split extends MultiMorph {

	public Split(GraphNode s, GraphNode t1, GraphNode t2) {
		super(s);
		targets = new GraphNode[2];
		targets[0] = t1;
		targets[1] = t2;
	}
	
	public void init() {
		if (sources == null)
			sources = new GraphNode[2];
		new_edges = new Vector();
		old_edges = new Vector();
		internal_edges = new Vector();

		for (int i = 0; i < 2; i++) {
			sources[i] = (GraphNode)source.clone();
			sources[i].attributes = targets[i].attributes != null ?
				(Attributes)targets[i].attributes.clone() :
				null;
		}

		for (int i = 0; i < 2; i++) {
			for (int j = 0; j < targets[i].inDegree(); j++) {
				GraphEdge e = (GraphEdge)targets[i].getInEdge(j);
				GraphEdge ge = null;
				ge = getEdge_SourceSignificant((GraphNode)e.source, source, e.label);
				if (ge == null && e.source == e.target)
					ge = getEdge_SourceSignificant(source, source, e.label);
				if (ge != null) {
					// An edge like this one was previously leading to the
					// unsplit node
					GraphEdge cloned_edge = (GraphEdge)ge.clone();
					cloned_edge.source = ge.source;
					if (e.source == e.target)
						cloned_edge.source = sources[i];
					cloned_edge.target = sources[i];
					cloned_edge.attributes = ge.attributes != null ?
						(Attributes)e.attributes.clone() : null;
					sources[i].addInEdge(cloned_edge);
					if (ge.source == ge.target)
						sources[i].addOutEdge(cloned_edge);
				} else {
					// There was previously no edge like this one leading to
					// the unsplit node.
					if (e.source == targets[1 - i])
						internal_edges.addElement(e);
					else
						new_edges.addElement(e);
				}
			}
			for (int j = 0; j < targets[i].outDegree(); j++) {
				GraphEdge e = (GraphEdge)targets[i].getOutEdge(j);
				if (e.source == e.target)
					continue;
				GraphEdge ge = null;
				ge = getEdge_TargetSignificant(source, (GraphNode)e.target, e.label);
				if (ge != null) {
					// An edge like this one was previously leaving the
					// unsplit node
					GraphEdge cloned_edge = (GraphEdge)ge.clone();
					cloned_edge.source = sources[i];
					cloned_edge.target = ge.target;
					cloned_edge.attributes = ge.attributes != null ?
						(Attributes)e.attributes.clone() : null;
					sources[i].addOutEdge(cloned_edge);
				} else {
					// There was previously no edge like this one leading to
					// the unsplit node.
					if (e.target != targets[1 - i])
						new_edges.addElement(e);
				}
			}
		}

		for (int i = 0; i < source.inDegree(); i++) {
			GraphEdge e = (GraphEdge)source.getInEdge(i);
			if ((
				e.source != e.target
				 	&&
				getEdge_SourceSignificant((GraphNode)e.source, targets[0], e.label) == null
				 	&&
				getEdge_SourceSignificant((GraphNode)e.source, targets[1], e.label) == null
				) || (
				e.source == e.target
					&&
				getEdge_SourceSignificant(targets[0], targets[0], e.label) == null
					&&
				getEdge_SourceSignificant(targets[1], targets[1], e.label) == null
				))
			{
				// There is no longer an edge like this one leading to
				// one of the split nodes.
				old_edges.addElement(e);
			}
		}

		for (int i = 0; i < source.outDegree(); i++) {
			GraphEdge e = (GraphEdge)source.getOutEdge(i);
			if (e.source != e.target
				&&
				getEdge_SourceSignificant(targets[0], (GraphNode)e.target, e.label) == null
				&&
				getEdge_SourceSignificant(targets[1], (GraphNode)e.target, e.label) == null)
			{
				// There is no longer an edge like this one leaving
				// one of the split nodes.
				old_edges.addElement(e);
			}
		}

/*		System.out.println("Summary of initSplit: ");
		Vector v = new_edges;
		if (v.size() > 0)
			System.out.println("New edges:");
		for (int j = 0; j < v.size(); j++)
			System.out.println("  " + v.elementAt(j));
		if (old_edges.size() > 0)
			System.out.println("Old edges:");
		for (int j = 0; j < old_edges.size(); j++)
			System.out.println("  " + old_edges.elementAt(j));
		if (internal_edges.size() > 0)
			System.out.println("new internal edges:");
		for (int j = 0; j < internal_edges.size(); j++)
			System.out.println("  " + internal_edges.elementAt(j));
*/
	}
}