package alexsa.data.tvp;

import java.util.Vector;
import alexsa.data.graphs.ShapeGraph;
import alexsa.data.Edge;
import alexsa.data.Node;
import alexsa.data.graphs.GraphNode;
import alexsa.gui.codeview.DisplayLine;

public class CfgNode extends Node {
	public Vector
		shape_graphs = null;

	public CfgNode(String id) {
		name = id;
	}

	public CfgNode(Node n) {
		this(n.name);
		color = n.color;
	}

	public void addShapeGraph(ShapeGraph g) {
		if (shape_graphs == null)
			shape_graphs = new Vector();
		shape_graphs.addElement(g);
	}
	
	public ShapeGraph getPreviousShapeGraph(ShapeGraph g) {
		if (shape_graphs == null)
			return null;
		int i = shape_graphs.indexOf(g);
		return (i > 0) ? (ShapeGraph)shape_graphs.elementAt(i - 1) : null;
	}
	
	public ShapeGraph getNextShapeGraph(ShapeGraph g) {
		if (shape_graphs == null)
			return null;
		int i = shape_graphs.indexOf(g);
		return (i < shape_graphs.size() - 1) ? (ShapeGraph)shape_graphs.elementAt(i + 1) : null;
	}
	
	protected DisplayLine addLine(Vector lines, String label, String code, int x, Edge edge) {
		DisplayLine dl = new DisplayLine(label, code, x, (CfgEdge)edge);
		lines.addElement(dl);
		return dl;
	}
	
	protected DisplayLine addLine(Vector lines, String label, String code, int x, Edge edge, String go_to) {
		DisplayLine dl = new DisplayLine(label, code, x, (CfgEdge)edge, go_to);
		lines.addElement(dl);
		return dl;
	}
	
	public void createDisplayLines(Vector lines, int x) {
		if (color != Node.WHITE)
			return;

		color = Node.GRAY;
		for (int i = 0; i < outDegree(); i++) {
			Edge e = getOutEdge(i);
			// If the target has no outgoing edges, it is a final state
			// and can thus be modeled by an explicit "return"-statement.
			// We also have to check, whether the target node is a simple
			// CfgNode, as more complex nodex (like WhileLoop) don't have
			// outgoing edges either.
			if (e.target.outDegree() == 0 &&
				! (e.target instanceof alexsa.gui.codeview.ComplexNode))
			{
				addLine(lines, e.source.name,
					((CfgEdge)e).code + ";",
					x, e, null);
				addLine(lines, null, "return", x, null);
			} else
				addLine(lines, e.source.name,
					((CfgEdge)e).code + ";",
					x, e, e.target.name);
		}
		for (int i = 0; i < outDegree(); i++)
			((CfgNode)getOutEdge(i).target).createDisplayLines(lines, x);
		color = Node.BLACK;
	}
}
