package alexsa.data.tvp;

import java.util.Vector;

import alexsa.data.Tools;

public class CorePredicate extends Declaration {

	public Predicate
		predicate = null;
	public Vector
		variables = null;
	public Flags
		flags = null;

	public CorePredicate(Predicate p, Vector v, Flags f) {
		if (p == null)
			nullParameter("Core predicate", "a predicate");
		type = CORE_PREDICATE;
		predicate = p;
		variables = v;
		flags = f;
	}

	/* Copy-constructor, uses also copy-constructors for member variables */
	public CorePredicate(CorePredicate cp) {
		predicate = cp.predicate != null ? new Predicate(cp.predicate) : null;
		variables = cp.variables != null ? new Vector(cp.variables) : null;
		flags = cp.flags != null ? new Flags(cp.flags) : null;
	}


	public void replace(String a, String b) {
		if (predicate != null)
			predicate.replace(a, b);
		Tools.replaceInVector(variables, a, b);
		if (flags != null)
			flags.replace(a, b);
	}

	/* does this core predicate define a variable? A predicate is said to
	   define a variable, if its variable list is of length 1, if the predicate's
	   name has no id_list, if the predicate's name is identical to the String
	   s and if one of the flags is "unique"
	*/
	public boolean isVariable(String s) {
		return (variables != null && variables.size() == 1 &&
			predicate.id_list == null && predicate.name.equals(s) &&
			flags != null && flags.contains("unique"));
	}

	public String toString() {
		StringBuffer b = new StringBuffer();
		b.append("%p ");
		b.append(predicate);
		b.append('(');
		b.append(printVector(variables, ','));
		b.append(')');
		if (flags != null) {
			b.append(' ');
			b.append(flags);
		}
		return b.toString();
	}
}
