package alexsa.data.tvp;

import java.util.Vector;

/** This class implements some basic functions used by many of the
	classes representing elements of TVP-files.
*/
public class FileElement {

	protected void nullParameter(String class_name, String missing_item) {
		throw new NullPointerException(class_name + " requires " +
					   missing_item + ", but none was supplied.");
	}

	public void appendLine(StringBuffer b, Object o) {
		appendLine(b, null, o, null);
	}

	public void appendLine(StringBuffer b, String prefix, Object o) {
		appendLine(b, prefix, o, null);
	}

	public void appendLine(StringBuffer b, String prefix, Object o, String suffix) {
		if (prefix != null)
			b.append(prefix);
		b.append(o);
		if (suffix != null)
			b.append(suffix);
		b.append('\n');
	}

	/* returns the elements of v printed in order. If c != 0, 
	   the elements are separated by c.
	   If v == null or empty, the empty string "" is returned.
	*/
	public static String printVector(Vector v, char c) {
		if (v == null || v.size() == 0)
			return "";
		StringBuffer b = new StringBuffer();
		for (int i = 0; i < v.size() - 1; i++) {
			b.append(v.elementAt(i));
			if (c != 0)
				b.append(c);
			b.append(' ');
		}
		b.append(v.elementAt(v.size() - 1));

		return b.toString();
	}

	/* returns the elements of v printed in order. If s != null, 
	   the elements are separated by s.
	   If v == null or empty, the empty string "" is returned.
	*/
	public static String printVector(Vector v, String s) {
		if (v == null || v.size() == 0)
			return "";
		StringBuffer b = new StringBuffer();
		for (int i = 0; i < v.size() - 1; i++) {
			b.append(v.elementAt(i));
			if (s != null)
				b.append(s);
		}
		b.append(v.elementAt(v.size() - 1));

		return b.toString();
	}

	public static String printVector(Vector v) {
		return printVector(v, (char)0);
	}

}
