package alexsa.data.tvp;

import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;

import java.io.File;

import java_cup.runtime.*;
import alexsa.data.FileFormatException;
import alexsa.data.Tools;
import alexsa.data.animation.Trace;
import alexsa.data.graphs.ShapeGraph;
import alexsa.data.graphs.GraphNode;
import alexsa.data.Node;
import alexsa.gui.codeview.ComplexNode;

public class Program implements alexsa.Debug {

	private Vector
		// the main parts of the TVP file (in this order)
		declaration_list = null,
		action_list = null,
		cfg_edge_list = null;
	public Vector
		 // CFG-Nodes to be displayed
		dead_cfg_node_list = null;


	private Hashtable
		sets = null,
		core_predicates = null,
		instrumentation_predicates = null,
		consistency_rules = null,
		nodes = null,
		graphs2cfg_nodes;

	// first and last node of the TVP program
	public CfgNode
		start_node = null,
		end_node = null;
		
	public Vector
		// The vector of TVS structures as read from the TVS-file
		tvs = null;

	// The TVP file this program was read from
	public File
		file = null;
	
	// a possibly existing graph of the TVP-program
	public ShapeGraph
		cfg = null;
	
	public Program() {
	}

	public Program(Vector dl, Vector al, Vector cel, Vector cnl) {
		declaration_list = dl;
		action_list = al;
		cfg_edge_list = cel;
		dead_cfg_node_list = cnl;
	}

	public CfgNode getCfgNode(String id) {
		return (CfgNode)nodes.get(id);
	}

	public void replaceNode(CfgNode old_node, CfgNode new_node) {
		nodes.remove(old_node.name);
		nodes.put(new_node.name, new_node);
		for (int i = 0; i < old_node.inDegree(); i++)
			old_node.getInEdge(i).source.replaceInOutEdges(old_node, new_node);
		for (int i = 0; i < old_node.outDegree(); i++)
			old_node.getOutEdge(i).target.replaceInInEdges(old_node, new_node);
		new_node.shape_graphs = old_node.shape_graphs;
		if (new_node.shape_graphs != null)
			for (int i = 0; i < new_node.shape_graphs.size(); i++)
				graphs2cfg_nodes.put(new_node.shape_graphs.elementAt(i), new_node);
	}

	protected void init_declaration(Declaration d, String id, String iterated) {
		if (d == null) {
			if (DEBUG)
			System.err.println("Declaration == null!");
			return;
		}
		switch (d.type) {

			case Declaration.SET_DECLARATION:
				SetDeclaration sd = (SetDeclaration)d;
				sets.put(sd.set_id, sd.set);
				break;

			case Declaration.CORE_PREDICATE:
				CorePredicate cp = (CorePredicate)d;
				if (id != null) {
					cp = new CorePredicate(cp);
					cp.replace(id, iterated);
				}
				core_predicates.put(cp.predicate, cp);
				break;

			case Declaration.INSTRUMENTATION_PREDICATE:
				InstrumentationPredicate ip = (InstrumentationPredicate)d;
				if (id != null) {
					ip = new InstrumentationPredicate(ip);
					ip.replace(id, iterated);
				}
				instrumentation_predicates.put(ip.predicate, ip);
				break;

			case Declaration.CONSISTENCY_RULE:
				ConsistencyRule cr = (ConsistencyRule)d;
				if (id != null) {
					cr = new ConsistencyRule(cr);
					cr.replace(id, iterated);
				}
				consistency_rules.put(cr.formula_a, cr);
				break;

			case Declaration.FOREACH:
				ForEach fe = (ForEach)d;
				Iterator it = fe.iterator;
				Vector set = it.set_expression.eval(sets);
				for (int i = 0; i < set.size(); i++)
					init_declaration_list(fe.declaration_list, it.identifier, (String)set.elementAt(i));
				break;

			case Declaration.UNDETERMINED:
				if (DEBUG)
					System.err.println("Undetermined declaration");
				break;
		}
	}

	protected void init_declaration_list(Vector dl, String id, String iterated) {
		for (int i = 0; i < dl.size(); i++)
			init_declaration((Declaration)dl.elementAt(i), id, iterated);
	}

	protected void init_cfg_graph() {
		nodes = new Hashtable();
		/*	if (cfg_node_list != null)
			for (int i = 0; i < cfg_node_list.size(); i++) {
			String id = (String )cfg_node_list.elementAt(i);
			nodes.put(id, new CfgNode(id));
			}*/
		if (cfg_edge_list != null)
			for (int i = 0; i < cfg_edge_list.size(); i++) {
				CfgEdge ce = (CfgEdge)cfg_edge_list.elementAt(i);
				CfgNode
					a = (CfgNode)nodes.get(ce.node_a_id),
					b = (CfgNode)nodes.get(ce.node_b_id);
				if (a == null)
					nodes.put(ce.node_a_id, (a = new CfgNode(ce.node_a_id)));
				if (b == null)
					nodes.put(ce.node_b_id, (b = new CfgNode(ce.node_b_id)));
				ce.source = a;
				ce.target = b;
				a.addOutEdge(ce);
				b.addInEdge(ce);
				if (start_node == null)
					start_node = a;
			}

		// Now we try to determine the start and final state of the
		// control flow graph
		Enumeration e = nodes.elements();
		while (e.hasMoreElements()) {
			CfgNode c = (CfgNode)e.nextElement();
			if (c.inDegree() == 0 && c.outDegree() == 0)
				System.err.println("Error: node " + c + " is isolated.");
			else if (c.outDegree() == 0) {
				if (end_node != null)
					System.err.println("Non-critical warning: nodes " + end_node +
					   " and " + c +
					   " both seem to be final nodes.\n" +
					   "This is not necessarily error.");
				else
					end_node = c;
			}
		}
		if (DEBUG) {
			if (start_node != null)
				System.out.println("Start node: " + start_node);
			else
				System.out.println("no start node found");
			if (end_node != null)
				System.out.println("Final node: " + end_node);
			else
				System.out.println("no final node found");
		}

		if (start_node != null)
			check_connectivity(start_node);
		else
			System.out.println("Couldn't check for connectivity of control flow graph" +
				", as no start node could be identified.");
	}

	protected boolean check_connectivity(CfgNode start) {

		// now we check, whether the graph is connected. Only connected
		// graphs can yield correct results.
		Node.dfs(start);

		// now we test, whether the dfs was able to reach all nodes.
		boolean connected = true;
		Enumeration e = nodes.elements();
		while (e.hasMoreElements()) {
			CfgNode n = (CfgNode)e.nextElement();
			if (n.color == Node.WHITE) {
				System.err.println("node " + n +
				   " is not reachable from start node " + start_node);
				connected = false;
			}
		}
		return connected;
	}

	// we expect a graph to be a state label if it has exactly one
	// node and if its name is identical to that used in any one
	// cfg edge
	public boolean isStateLabel(ShapeGraph g) {
		if (g.getNumberOfNodes() != 1)
			return false;
		String name = g.getNode(0).name;
		for (int i = 0; i < cfg_edge_list.size(); i++) {
			CfgEdge ce = (CfgEdge)cfg_edge_list.elementAt(i);
			if (name.equals(ce.node_a_id) || name.equals(ce.node_b_id))
				return true;
		}
		return false;
	}
	
	/* is the given string the name of a variable?
		Result:
		  true: a unary core predicate with display flag "unique" has
		        been declared
		  false: if not true;
		This function does not need to evaluate *foreach*-declarations, as
		this has been done before in init().
	*/
	public boolean isVariable(String s) {
		Enumeration e = core_predicates.elements();
		while (e.hasMoreElements()) {
			CorePredicate cp = (CorePredicate)e.nextElement();
			if (cp.predicate.equals(s))
				return cp.isVariable(s);
		}
		return false;
	}
	
	/* this method associates the graphs from the dot-output with
		the nodes from the control flow graph specified in the .tvp-file.
		It does this assuming, that the graphs are arranged in such a way
		that there is always a graph containing only one node labeled
		with the name of cfg-node followed by n graphs representing the
		shape graphs associated with that particular cfg-node.
	*/
	public void combineNodesWithGraphs(Trace trace)
		throws FileFormatException
	{
		if (trace == null)
			return;
		graphs2cfg_nodes = new Hashtable();

		for (int i = 0; i < trace.locations.size(); i++) {
			Vector location = (Vector)trace.locations.elementAt(i);
			String name = (String)location.elementAt(0);
			CfgNode node = getCfgNode(name);
			for (int j = 1; j < location.size(); j++) {
				ShapeGraph g = (ShapeGraph)location.elementAt(j);
				node.addShapeGraph(g);
				graphs2cfg_nodes.put(g, node);
			}
		}
	}
	
	/* init is called once after the input file has been read in completely.
	   interpretes the read data and creates various fields accordingly.
	*/
	public void init() {
		sets = new Hashtable();
		core_predicates = new Hashtable();
		instrumentation_predicates = new Hashtable();
		consistency_rules = new Hashtable();

		if (declaration_list != null)
			init_declaration_list(declaration_list, null, null);

		init_cfg_graph();
	}

	public Vector getCfgEdgeList() {
		return cfg_edge_list;
	}

	public Hashtable getSets() {
		return (sets == null || sets.size() == 0) ? null : sets;
	}

	public Hashtable getCorePredicates() {
		return (core_predicates == null || core_predicates.size() == 0)
			? null
			: core_predicates;
	}

	public Hashtable getInstrumentationPredicates() {
		return (instrumentation_predicates == null || instrumentation_predicates.size() == 0)
			? null
			: instrumentation_predicates;
	}

	public Hashtable getConsistencyRules() {
		return (consistency_rules == null || consistency_rules.size() == 0)
			? null
			: consistency_rules;
	}

	public CfgNode getCfgNode(ShapeGraph g) {
		return (CfgNode)graphs2cfg_nodes.get(g);
	}

	public Action getAction(String identifier) {
		if (action_list == null)
			return null;
		for (int i = 0; i < action_list.size(); i++) {
			Action a = (Action)action_list.elementAt(i);
			if (a.identifier.equals(identifier))
				return a;
		}
		return null;
	}
	
	public Object[] getAllActions() {
		return action_list.toArray();
	}

	protected boolean mappable(Vector a, Vector b) {
		if (a.size() != b.size())
			return false;
		for (int i = a.size() - 1; i >= 0; i--) {
			String
				as = (String)a.elementAt(i),
				bs = (String)b.elementAt(i);
			if (a == null || b == null)
				return false;
			if (Character.isLetter(as.charAt(0))) {
				if ((as.equals("null") || as.equals("true") || as.equals("false") ||
					bs.equals("null") || bs.equals("true") || bs.equals("false"))
					 && !bs.equals(as))
					return false;
			} else if (!as.equals(bs))
					return false;
		}
		return true;
	}

	public Action getActionByTitle(String title) {
		if (action_list == null)
			return null;
		Vector title_parts = Tools.tokenize(title);
		for (int i = 0; i < action_list.size(); i++) {
			Action action = (Action)action_list.elementAt(i);
			if (action.title == null)
				continue;
			Vector action_title = Tools.tokenize(action.title.toString());
			if (mappable(title_parts, action_title))
				return action;
		}
		return null;
	}

	public Enumeration graphs() {
		return new AllGraphsEnumeration();
	}
	
	public Enumeration graphs(String node_identifier) {
		return new GraphEnumeration(node_identifier);
	}
	
	public Enumeration cfgNodes() {
		return nodes.elements();
	}
	
	public void resetCfgNodes() {
		Enumeration e = cfgNodes();
		while (e.hasMoreElements()) {
			Node n = (Node)e.nextElement();
			n.color = Node.WHITE;
		}
		
	}
	
	class AllGraphsEnumeration implements Enumeration {
		private Vector
			graphs = null;
		private int
			current = 0;
	
		public AllGraphsEnumeration() {
			graphs = new Vector();
			Enumeration e = nodes.elements();
			while (e.hasMoreElements()) {
				CfgNode cn = (CfgNode)e.nextElement();
				if (cn.shape_graphs != null)
					for (int j = 0; j < cn.shape_graphs.size(); j++)
						graphs.addElement(cn.shape_graphs.elementAt(j));
			}
			current = 0;
		}
		
		public boolean hasMoreElements() {
			return graphs != null && current < graphs.size();
		}
		
		public Object nextElement() {
			return hasMoreElements() ? graphs.elementAt(current++) : null;
		}
	}

	class GraphEnumeration implements Enumeration {
		private Vector
			graphs = null;
		private int
			current = 0;
	
		public GraphEnumeration(String node_identifier) {
			graphs = new Vector();
			CfgNode cn = getCfgNode(node_identifier);
			if (cn.shape_graphs != null)
				for (int j = 0; j < cn.shape_graphs.size(); j++)
					graphs.addElement(cn.shape_graphs.elementAt(j));
			current = 0;
		}
		
		public boolean hasMoreElements() {
			return graphs != null && current < graphs.size();
		}
		
		public Object nextElement() {
			return hasMoreElements() ? graphs.elementAt(current++) : null;
		}
	}
}
