package alexsa.data.tvs;

import java.util.Vector;

import alexsa.data.tvp.FileElement;
import alexsa.data.tvp.Kleene;
import alexsa.data.tvp.Predicate;
import alexsa.data.tvp.Program;

public class PredicateAssignment {
	public Predicate
		name = null;
	public Kleene
		nullary = null;
	public Vector
		unary = null,
		binary = null;
	
	public PredicateAssignment(Predicate p, Kleene k) {
		name = p;
		nullary = k;
	}
	
	public PredicateAssignment(Predicate p, Vector v) {
		name = p;
		if (v == null)
			return;
		if (v.size() == 0)
			unary = v;
		else {
			Object o = v.firstElement();
			if (o instanceof NodePair)
				binary = v;
			else if (o instanceof TvsNode)
				unary = v;
		}
	}
	
	public boolean isSummarized() {
		return name.name.equals("sm");
	}
	
	public int size() {
		if (unary == null && binary == null)
			return 0;
		else
			return unary == null ? binary.size() : unary.size();
	}
	
	/* Condition for this predicate declaring an element of the universe
	   to be a variable:
	    - The predicate's name must be the name of a variable declared in
	      the TVP program
	    - The list of unary predicate values has length 1
	    - The predicates value is a member of the universe
	*/
	public boolean isVariable(Vector universe, Program tvp) {
		if (name.id_list != null ||
			universe == null ||
			universe.size() == 0 ||
			unary == null ||
			unary.size() != 1)
		{
			return false;
		}
		String node_name = ((TvsNode)unary.firstElement()).id;
		boolean universe_member = false;
		for (int i = 0; i < universe.size() && !universe_member; i++)
			universe_member = ((TvsNode)universe.elementAt(i)).id.equals(node_name);
		return universe_member && tvp.isVariable(name.name);
	}

	public String toString() {
		if (nullary != null)
			return name.toString() + " = {" + nullary + "}";
		else if (unary != null)
			return name.toString() + " = {" + FileElement.printVector(unary, ',') + "}";
		else if (binary != null)
			return name.toString() + " = {" + FileElement.printVector(binary, ',') + "}";
		else
			return name.toString() + " (erroneous)";
	}
}
