package alexsa.data.tvs;

import alexsa.data.tvp.FileElement;
import alexsa.data.tvp.Program;
import java.util.Vector;

public class Structure {
	private static int
		global_id = 0;
	private int
		local_id;
	public Vector
		universe = null,
		predicates = null;
	public String
		description = null;
	
	public Structure(Vector u, Vector p) {
		this(u, p, null);
	}
	
	public Structure(Vector u, Vector p, String d) {
		universe = u;
		predicates = p;
		description = d;
		local_id = ++global_id;
	}
	
	public String toString() {
		return "%n = {" + FileElement.printVector(universe, ',') +
			"}\n%p = {\n\t" + FileElement.printVector(predicates, "\n\t") +"\n}";
	}

	/* do some analysis of this structure. This is used to give
		a rough textual description of this structure whenever the user
		is supposed to select a structure. The textual representations
		should enable the user to identify the structures easier than just
		by their integer ID.
		Depending on the analysis, the structure can have a brief (less than
		40 char) or longer textual analysis.
	*/
	protected String
		short_analysis = null,
		long_analysis = null;

	public void analyze(Program tvp) {
		createLongAnalysis(tvp);
		createShortAnalysis();
	}
	
	protected void createLongAnalysis(Program tvp) {
		int summarized_cells = 0;
		Vector vars = new Vector();
		for (int i = 0; i < predicates.size(); i++) {
			PredicateAssignment p = (PredicateAssignment)predicates.elementAt(i);
			if (p.isVariable(universe, tvp))
				vars.addElement(p.name.name);
			if (p.isSummarized())
				summarized_cells += p.size();
		}
		StringBuffer buf = new StringBuffer();
		buf.append("Shapegraph #" + local_id);
		if (universe == null || universe.size() == 0)
			buf.append(" is empty");
		else {
			if (vars.size() > 0) {
 				buf.append(" contains the ");
 				if (vars.size() == 1)
					buf.append("variable " + vars.elementAt(0));
				else {
					buf.append(vars.size() + " variables ");
					for (int i = 0; i < (vars.size() - 2); i++) {
						buf.append(vars.elementAt(i));
						buf.append(", ");
					}
					buf.append(vars.elementAt(vars.size() - 2) + " and " +
						vars.elementAt(vars.size() - 1));
				}
				buf.append(",\n  ");
			} else
				buf.append(" has no variables,\n  ");
			int heap_cells = universe != null ? universe.size() : 0;
			if (heap_cells > 0) {
				if (heap_cells == 1) {
					if (summarized_cells == 1)
						buf.append("and one summarized heap cell."); 
					else
						buf.append("and one heap cell.");
				} else {
					if (heap_cells - summarized_cells == 0)
						buf.append("and " + printQuantity(summarized_cells, "summarized cell") + '.');
					else {
						buf.append(printQuantity((heap_cells - summarized_cells), "heap cell"));
						buf.append(" and " + printQuantity(summarized_cells, "summarized cell") + '.');
					}
				}
			} else {
				buf.append("and no allocated heap cells.");
			}
		}
		buf.append('\n');
		long_analysis = buf.toString();
	}
	
	/* while the long analysis (see above) creates a statistic, the short
		analysis tries to categorize the shape graph.
	*/
	protected void createShortAnalysis() {
/*		// first: list of length 1?
		if (oneHeapCellAllocated())
			short_analysis = "One allocated heap cell";
		// second; list of length > 1?
		else if (listLengthGreaterOne())
			short_analysis = "List of length > 1";*/
	}

/*	// does this shape graph describe exactly one allocated heap cell?
	protected boolean oneHeapCellAllocated() {
		if (nodes.size() != 2 || edges.size() != 1)
			return false;
		GraphNode
			var = null,
			cell = null,
			one, two;
		one = (GraphNode)nodes.elementAt(0);
		two = (GraphNode)nodes.elementAt(1);
		if (one.getShape() == GraphSymbols.BOX && two.getShape() == GraphSymbols.ELLIPSE) {
			var = one;
			cell = two;
		}
		if (one.getShape() == GraphSymbols.ELLIPSE && two.getShape() == GraphSymbols.BOX) {
			var = two;
			cell = one;
		}
		GraphEdge edge = (GraphEdge)edges.elementAt(0);
		return (var != null && cell != null &&
			edge.source_name.equals(var.name) &&
			edge.target_name.equals(cell.name) &&
			!cell.isDotted() &&
			!edge.isDotted());
	}
	*/
	/* check: is there exactly one variable (box node), one
		non-summarized heap-cell (solid ellipse) and one
		summarized node (dotted ellipse), plus
		one solid edge from the variable to the start of the list,
		one dotted edge to the rest of the list and one dotted
		edge internal to the rest of the list?
	*/ 
/*	protected boolean listLengthGreaterOne() {
		if (nodes.size() != 3 || edges.size() != 3)
			return false;
		GraphNode
			var = null,
			start = null,
			rest = null;
		for (int i = 0; i < 3; i++) {
			GraphNode n = (GraphNode)nodes.elementAt(i);
			if (n.getShape() == GraphSymbols.BOX) {
				if (var != null)
					return false;
				else
					var = n;
			} else if (n.getShape() == GraphSymbols.ELLIPSE) {
				if (n.isDotted()) {
					if (rest != null)
						return false;
					else
						rest = n;
				} else {
					if (start != null)
						return false;
					else
						start = n;
				}
			}
		}
		if (var == null || start == null || rest == null)
			return false;

		GraphEdge
			allocation = null,
			further = null,
			internal = null;
		for (int i = 0; i < 3; i++) {
			GraphEdge e = (GraphEdge)edges.elementAt(i);
			if (!e.isDotted() && e.source_name.equals(var.name) &&
				e.target_name.equals(start.name))
			{
				if (allocation != null)
					return false;
				else
					allocation = e;
			} else if (e.isDotted() && e.source_name.equals(start.name) &&
				e.target_name.equals(rest.name))
			{
				if (further != null)
					return false;
				else
					further = e;
			} else if (e.isDotted() && e.source_name.equals(rest.name) &&
				e.target_name.equals(rest.name))
			{
				if (internal != null)
					return false;
				else
					internal = e;
			}
		}
		return allocation != null && further != null && internal != null;
	}
*/
	protected String printQuantity(int i, String suffix) {
		if (i == 0)
			return "no " + suffix;
		else if (i == 1)
			return "one " + suffix;
		else
			return "" + i + ' ' + suffix + "s";
	}
	
	public String getShortAnalysis() {
		return short_analysis;
	}
	
	public String getLongAnalysis() {
		return long_analysis;
	}
}