package alexsa.gui.codeview;

import java.util.Vector;

import alexsa.data.tvp.CfgEdge;
import alexsa.data.tvp.CfgNode;
import alexsa.data.tvp.Program;
import alexsa.data.Edge;
import alexsa.data.Node;

class DoWhileLoop extends ComplexNode {
	CfgEdge
		while_condition = null,
		goto_on_exit = null;

	DoWhileLoop(Node n, Program program, CfgEdge back_edge_out) {
		super(n, program);
		while_condition = back_edge_out;
		goto_on_exit = (CfgEdge)edgeNotPointingFromTo(n, back_edge_out.target);

		for (int i = 0; i < back_edge_out.target.inDegree(); i++) {
			Edge e = back_edge_out.target.getInEdge(i);
			if (e.source != this)
				e.target = this;
		}

		if (back_edge_out.target == program.start_node)
			program.start_node = this;
	}

	public void replaceInOutEdges(Node old_node, Node new_node) {
		super.replaceInOutEdges(old_node, new_node);
		if (while_condition.target == old_node)
			while_condition.target = new_node;
		if (goto_on_exit.target == old_node)
			goto_on_exit.target = new_node;
	}

	public Edge edgeNotPointingFromTo(Node from, Node to) {
		if (from.outDegree() != 2)
			return null;
		Edge e0 = from.getOutEdge(0),
			e1 = from.getOutEdge(1);
		if (e0.target == to && e1.target != to)
			return e1;
		if (e0.target != to && e1.target == to)
			return e0;
		return null;
	}

	public void createDisplayLines(Vector lines, int x) {
		if (color != Node.WHITE)
			return;
		color = Node.GRAY;
		DisplayLine do_line = addLine(lines, null, "do {", x, null);
			((CfgNode)while_condition.target).createDisplayLines(lines, x + 1);
		do_line.detour_index = lines.size();
		addLine(lines, name, "} while (" + while_condition.code + ");", x, while_condition);
		((CfgNode)goto_on_exit.target).createDisplayLines(lines, x);
		color = Node.BLACK;
	}
}
