package alexsa.gui.dialogs;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import alexsa.Preferences;
import alexsa.data.animation.Analysis;

public class AnimationPrefsDialog extends JDialog {
	public class PriorityModel implements ListModel {

		private final static String
			EXTERNAL_LABEL	= "External cycles",
			INTERNAL_LABEL	= "Internal cycles",
			BRANCHED_LABEL	= "Number of branchings",
			STRAIGHT_LABEL	= "Depth",
			FORWARD_LABEL	= "Forward edges",
			CROSS_LABEL		= "Cross edges";

		AnimationPrefsDialog dialog = null;

		public PriorityModel(AnimationPrefsDialog apd) {
			dialog = apd;
		}
		
		public void addListDataListener(ListDataListener L) {
		}

		public void removeListDataListener(ListDataListener L) {
		}

		public int getSize() {
			return number_of_priorities;
		}

		public Object getElementAt(int i) {
			switch (dialog.order[i]) {
				case Analysis.STRAIGHT:
					return STRAIGHT_LABEL;
				case Analysis.INTERNAL_CYCLE:
					return INTERNAL_LABEL;
				case Analysis.EXTERNAL_CYCLE:
					return EXTERNAL_LABEL;
				case Analysis.BRANCHED:
					return BRANCHED_LABEL;
				case Analysis.CROSS_EDGING:
					return CROSS_LABEL;
				case Analysis.FORWARD_EDGING:
					return FORWARD_LABEL;
			}
			return "??";
		}
		
	}
	
	public JTextField
		master_speed = null,
		zoom_steps	= null,
		zoom_delay	= null,
		remove_steps = null,
		remove_delay = null,
		create_steps = null,
		create_delay = null,
		move_steps	= null,
		move_delay	= null,
		merge_steps	= null,
		merge_delay	= null,
		split_steps	= null,
		split_delay	= null,
		final_delay	= null,
		
		number_of_cycles = null;
	
	public JList
		priorities = null;
	
	private final static int
		number_of_priorities = 6;
	public int[]
		order = new int[number_of_priorities];

	
	public AnimationPrefsDialog() {
		super((Frame)null, "Animation preferences", true);
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		Container content = getContentPane();
		content.setLayout(new GridBagLayout());
		GridBagConstraints outer_gbc = new GridBagConstraints();
		outer_gbc.gridx = outer_gbc.gridy = 0;
		outer_gbc.gridwidth = outer_gbc.gridheight = 1;
		outer_gbc.fill = GridBagConstraints.BOTH;
		outer_gbc.weightx = 1;
		outer_gbc.insets = new Insets(5, 5, 0, 5);
		
		// Visual execution
		JPanel visual = new JPanel(new GridBagLayout());
		visual.setBorder(new TitledBorder("Visual execution settings"));
		
		GridBagConstraints GBC = new GridBagConstraints();
		GBC.fill = GridBagConstraints.BOTH;
		GBC.gridx = GBC.gridy = 0;
		GBC.gridwidth = 2;
		GBC.gridheight = 1;
		GBC.weightx = 1;
		GBC.insets = new Insets(5, 10, 0, 10);
		visual.add(new JLabel("Number of cycle passes:"), GBC);
		number_of_cycles = new JTextField("" + Preferences.number_of_cycles, 3);
		GBC.gridx = 2;
		GBC.weightx = 0;
		visual.add(number_of_cycles, GBC);
		
		GBC.gridx = 0;
		GBC.gridy = 1;
		GBC.gridwidth = 3;
		visual.add(new JLabel("Priority of phenomena:"), GBC);

		GBC.gridx = 0;
		GBC.gridy = 2;
		GBC.gridwidth = 1;
		GBC.insets = new Insets(5, 25, 0, 10);
		JButton higher = new JButton("Higher");
		visual.add(higher, GBC);
		higher.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				int i = priorities.getSelectedIndex();
				if (i > 0) {
					int x = order[i];
					order[i] = order[i - 1];
					order[i - 1] = x;
					priorities.setSelectedIndex(i - 1);
				}
			}
		});
		GBC.gridy = 3;
		JButton lower = new JButton("Lower");
		lower.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				int i = priorities.getSelectedIndex();
				if (i >= 0 && i < number_of_priorities - 1) {
					int x = order[i];
					order[i] = order[i + 1];
					order[i + 1] = x;
					priorities.setSelectedIndex(i + 1);
				}
			}
		});
		visual.add(lower, GBC);
		GBC.gridy = 4;
		GBC.weighty = 1;
		visual.add(new JLabel(" "), GBC);
		
		GBC.gridx = 1;
		GBC.gridy = 2;
		GBC.gridheight = 3;
		GBC.gridwidth = 2;
		GBC.weightx = 1;
		GBC.insets = new Insets(5, 10, 0, 10);
		for (int i = 0; i < number_of_priorities; i++)
			order[i] = Preferences.order[i];
		priorities = new JList(new PriorityModel(this));
		priorities.setBorder(new LineBorder(Color.black, 1));
		visual.add(priorities, GBC);

		content.add(visual, outer_gbc);
		// End Visual execution

		outer_gbc.gridy = GridBagConstraints.RELATIVE;

		// Inbetweening
		JPanel inbetweening = new JPanel(new GridBagLayout());
		inbetweening.setBorder(new TitledBorder("Inbetweening settings"));
		
		GBC.gridx = GBC.gridy = 0;
		GBC.gridwidth = 3;
		GBC.gridheight = 1;
		inbetweening.add(new JLabel("Master speed factor:"), GBC);
		master_speed = new JTextField("" + Preferences.MASTER_SPEED, 5);
		GBC.gridx = 3;
		inbetweening.add(master_speed, GBC);
		
		GBC.gridx = 1;
		GBC.gridy = 1;
		GBC.gridwidth = 1;
		inbetweening.add(new JLabel("Steps"), GBC);
		GBC.gridx = 2;
		GBC.gridwidth = 2;
		inbetweening.add(new JLabel("Delay (x M.s.f. ms)"), GBC);

		GBC.gridx = 0;
		GBC.gridy = 2;
		GBC.weightx = 1;
		GBC.gridwidth = 1;
		inbetweening.add(new JLabel("Zooming:"), GBC);
		GBC.gridy = GridBagConstraints.RELATIVE;
		inbetweening.add(new JLabel("Creating:"), GBC);
		inbetweening.add(new JLabel("Removing:"), GBC);
		inbetweening.add(new JLabel("Moving:"), GBC);
		inbetweening.add(new JLabel("Merging:"), GBC);
		inbetweening.add(new JLabel("Splitting:"), GBC);
		GBC.gridwidth = 2;
		inbetweening.add(new JLabel("Final delay:"), GBC);
		
		GBC.gridx = 1;
		GBC.gridy = 2;
		GBC.gridwidth = 1;
		GBC.weightx = 0;
		zoom_steps = new JTextField("" + Preferences.ZOOM_STEPS, 3);
		inbetweening.add(zoom_steps, GBC);
		GBC.gridy = GridBagConstraints.RELATIVE;
		create_steps = new JTextField("" + Preferences.CREATE_STEPS, 3);
		inbetweening.add(create_steps, GBC);
		remove_steps = new JTextField("" + Preferences.REMOVE_STEPS, 3);
		inbetweening.add(remove_steps, GBC);
		move_steps = new JTextField("" + Preferences.MOVE_STEPS, 3);
		inbetweening.add(move_steps, GBC);
		merge_steps = new JTextField("" + Preferences.MERGE_STEPS, 3);
		inbetweening.add(merge_steps, GBC);
		split_steps = new JTextField("" + Preferences.SPLIT_STEPS, 3);
		inbetweening.add(split_steps, GBC);

		GBC.gridx = 2;
		GBC.gridy = 2;
		zoom_delay = new JTextField("" + Preferences.ZOOM_DELAY, 5);
		inbetweening.add(zoom_delay, GBC);
		GBC.gridy = GridBagConstraints.RELATIVE;
		create_delay = new JTextField("" + Preferences.CREATE_DELAY, 5);
		inbetweening.add(create_delay, GBC);
		remove_delay = new JTextField("" + Preferences.REMOVE_DELAY, 5);
		inbetweening.add(remove_delay, GBC);
		move_delay = new JTextField("" + Preferences.MOVE_DELAY, 5);
		inbetweening.add(move_delay, GBC);
		merge_delay = new JTextField("" + Preferences.MERGE_DELAY, 5);
		inbetweening.add(merge_delay, GBC);
		split_delay = new JTextField("" + Preferences.SPLIT_DELAY, 5);
		inbetweening.add(split_delay, GBC);

		final_delay = new JTextField("" + Preferences.FINAL_DELAY, 5);
		inbetweening.add(final_delay, GBC);

		content.add(inbetweening, outer_gbc);
		// End Inbetweening

		// Buttons
		JPanel button_panel = new JPanel(new FlowLayout());
		JButton ok = new JButton("OK");
		ok.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				Preferences.MASTER_SPEED = Integer.parseInt(master_speed.getText());
				Preferences.ZOOM_STEPS = Integer.parseInt(zoom_steps.getText());
				Preferences.ZOOM_DELAY = Integer.parseInt(zoom_delay.getText());
				Preferences.REMOVE_STEPS = Integer.parseInt(remove_steps.getText());
				Preferences.REMOVE_DELAY = Integer.parseInt(remove_delay.getText());
				Preferences.CREATE_STEPS = Integer.parseInt(create_steps.getText());
				Preferences.CREATE_DELAY = Integer.parseInt(create_delay.getText());
				Preferences.MOVE_STEPS = Integer.parseInt(move_steps.getText());
				Preferences.MOVE_DELAY = Integer.parseInt(move_delay.getText());
				Preferences.MERGE_STEPS = Integer.parseInt(merge_steps.getText());
				Preferences.MERGE_DELAY = Integer.parseInt(merge_delay.getText());
				Preferences.SPLIT_STEPS = Integer.parseInt(split_steps.getText());
				Preferences.SPLIT_DELAY = Integer.parseInt(split_delay.getText());
				Preferences.FINAL_DELAY = Integer.parseInt(final_delay.getText());
				
				boolean change_in_animation = false;
				int temp = Integer.parseInt(number_of_cycles.getText());
				if (temp != Preferences.number_of_cycles)
					change_in_animation = true;
				Preferences.number_of_cycles = temp;
				for (int i = 0; i < number_of_priorities; i++) {
					if (order[i] != Preferences.order[i])
						change_in_animation = true;
					Preferences.order[i] = order[i];
				}
				setVisible(false);
				if (change_in_animation)
					JOptionPane.showMessageDialog(null,
						"You have changed the preferences for the visual execution.\n" +
						"These changes will take effect with every newly opened file,\n" +
						"but they don't affect currently open files.",
						"Change in animation prefs", JOptionPane.INFORMATION_MESSAGE, null);
			}
		});
		
		JButton cancel = new JButton("Cancel");
		button_panel.add(ok);
		
		button_panel.add(cancel);
		cancel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				setVisible(false);
			}
		});
		
		content.add(button_panel, outer_gbc);
		// End Buttons
		
		pack();
		setLocation(100,100);
		setVisible(true);
	}
	
}
