package alexsa.gui.dialogs;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import alexsa.Preferences;

public class CodeViewPrefsDialog extends JDialog {
	public JTextField
		indention_string = null,
		startup_path = null;
	public JCheckBox
		summarize_unknown = null;
	
	public CodeViewPrefsDialog() {
		super((Frame)null, "Code view preferences", true);
		Container content = getContentPane();
		content.setLayout(new GridBagLayout());
		GridBagConstraints outer_gbc = new GridBagConstraints();
		outer_gbc.gridx = outer_gbc.gridy = 0;
		outer_gbc.gridwidth = outer_gbc.gridheight = 1;
		outer_gbc.fill = GridBagConstraints.BOTH;
		outer_gbc.weightx = 1;
		outer_gbc.insets = new Insets(5, 5, 0, 5);
		
		// Code view
		JPanel code_view = new JPanel(new GridBagLayout());
		code_view.setBorder(new TitledBorder("Code view settings"));
		
		GridBagConstraints GBC = new GridBagConstraints();
		GBC.fill = GridBagConstraints.BOTH;
		GBC.gridx = GBC.gridy = 0;
		GBC.gridwidth = GBC.gridheight = 1;
		GBC.anchor = GridBagConstraints.NORTHWEST;
		GBC.insets = new Insets(5, 10, 0, 10);
		GBC.weightx = 1;
		code_view.add(new JLabel("Indention:"), GBC);
		GBC.weightx = 0;
		indention_string = new JTextField(encodeIndention(Preferences.code_view_indention), 30);
		GBC.gridx = 1;
		code_view.add(indention_string, GBC);
				
		summarize_unknown = new JCheckBox("Summarize \"unknown\" branches", Preferences.summarize_unknown_branches);
		GBC.gridx = 0;
		GBC.gridy = 1;
		GBC.gridwidth = 2;
		GBC.weightx = 1;
		code_view.add(summarize_unknown, GBC);

		content.add(code_view, outer_gbc);
		// End Code view
		outer_gbc.gridy = GridBagConstraints.RELATIVE;

		// misc
		JPanel misc = new JPanel(new GridBagLayout());
		misc.setBorder(new TitledBorder("Miscellaneous settings"));
		GBC.gridx = GBC.gridy = 0;
		GBC.gridwidth = 1;
		
		misc.add(new JLabel("Startup path: "), GBC);
		GBC.weightx = 0;
		startup_path = new JTextField(Preferences.startup_path, 30);
		GBC.gridx = 1;
		misc.add(startup_path, GBC);

		content.add(misc, outer_gbc);
		// End misc


		// Buttons
		JPanel button_panel = new JPanel(new FlowLayout());
		JButton ok = new JButton("OK");
		ok.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				Preferences.code_view_indention = decodeIndention(indention_string.getText());
				Preferences.startup_path = startup_path.getText();
				
				boolean change_unknown = false;
				if (summarize_unknown.isSelected() != Preferences.summarize_unknown_branches)
					change_unknown = true;
				Preferences.summarize_unknown_branches = summarize_unknown.isSelected();

				setVisible(false);
				if (change_unknown)
					JOptionPane.showMessageDialog(null,
						"You have changed the preference for the summarization\n" +
						"of \'unknown\' branches. This change will take effect\n" +
						"for every newly opened file, but it doesn't affect\n" +
						"files currently open.",
						"Change in code view prefs", JOptionPane.INFORMATION_MESSAGE, null);
			}
		});
		
		JButton cancel = new JButton("Cancel");
		button_panel.add(ok);
		
		button_panel.add(cancel);
		cancel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				setVisible(false);
			}
		});
		
		content.add(button_panel, outer_gbc);
		// End Buttons
		
		pack();
		setLocation(100,100);
		setVisible(true);
	}
	
	// encode Tab-characters as \t
	private String encodeIndention(String raw) {
		StringBuffer b = new StringBuffer();
		for (int i = 0; i < raw.length(); i++) {
			if (raw.charAt(i) == '\t')
				b.append("\\t");
			else
				b.append(raw.charAt(i));
		}
		return b.toString();
	}

	// decode \t occurrences within the string to actual Tab-characters
	private String decodeIndention(String encoded) {
		StringBuffer b = new StringBuffer();
		for (int i = 0; i < encoded.length(); i++) {
			if (encoded.charAt(i) == '\\' &&
				i < encoded.length() - 1 &&
				encoded.charAt(i + 1) == 't')
			{
				b.append("\t");
				i++;
			} else
				b.append(encoded.charAt(i));
		}
		return b.toString();
	}
}
