/*	alexsa.gui.FMenuItem

	The main purpose of this class is to simplify menu-handling.
	Instead of remembering all JMenuItems in the main class and then
	testing for equality with various JMenuItems in case of an Event,
	we add a (distinct) integer value to every item, enabling us to
	find the appropriate handler for a command with a simple
	switch-statement. Thanks to Daniel Bobbert for this approach.
	
	Also, the constructors take more arguments, thus further
	reducing the clutter of statements usually needed when
	constructing the menubar.

	Ronald Bieber, robi@coli.uni-sb.de, May 20th 2000
*/

package alexsa.gui.menus;

import javax.swing.JMenuItem;
import javax.swing.KeyStroke;
import java.awt.event.ActionListener;

public class FMenuItem extends JMenuItem
{
	public FMenuItem(String cmdName, int cmdNum, KeyStroke cmdKey, ActionListener l, boolean State) {
		super(cmdName);
		if (cmdKey != null)
			setAccelerator(cmdKey);
		
		setActionCommand(Integer.toString(cmdNum));
		addActionListener(l);
		setEnabled(State);
	}
	public FMenuItem(String cmdName, int cmdNum, KeyStroke cmdKey, ActionListener l) {
		this(cmdName, cmdNum, cmdKey, l, false);
	}

	public FMenuItem(String cmdName, int cmdNum, int cmdKey, ActionListener l, boolean State)
	{
		this(cmdName, cmdNum, null, l, State);
		if (cmdKey  > 0)
			setAccelerator(KeyStroke.getKeyStroke(cmdKey, java.awt.Event.CTRL_MASK));
	}

	public FMenuItem(String cmdName, int cmdNum, int cmdKey, ActionListener l)
	{
		this(cmdName, cmdNum, cmdKey, l, false);
	}

	public int getCommandNumber()
	{
		return Integer.parseInt(getActionCommand());
	}
}
